*----------------------------------------------------------------------
*-- Program...: DBLOAT.H
*-- Programmer: Ken Mayer -- based on code by a lot of contributors
*-- Date......: 02/15/1997
*-- Notes.....: This file contains a series of "inline" functions or
*--             other routines. These are defined with the #define
*--             statement, and are fairly simple. In some cases,
*--             you may find more involved, or perhaps "expanded"
*--             versions of the same functions elsewhere in the dUFLP
*--             library. The comments are shortened as well, to keep
*--             the size of this file small.
*----------------------------------------------------------------------

*----------------------------------------------------------------------
*-- Misc. Definitions:
*----------------------------------------------------------------------
*-- CR = Carriage Return:
#define CR chr(10)

*----------------------------------------------------------------------
*-- LF = Line Feed:
#define LF chr(13)

*----------------------------------------------------------------------
*-- CRLF = Carriage Return/Line Feed:
#define chr(10)+chr(13)

*----------------------------------------------------------------------
*-- date routines
*----------------------------------------------------------------------
*-- Age -- Jay Parsons, Martin Leon and others -- returns numeric
*--        value, age (years) based on current date
*-- Usage:   age( dDate )
*-- Returns: numeric
#ifndef AGE
   #define AGE(dDate)           ( floor( ( val( dtos( date() ) ) ;
                                - val( dtos( dDate ) ) ) / 10000 ) )
#endif

*---------------------------------------------------------------------
*-- Age2 -- Ken Chan, Martin Leon, and others -- returns numeric
*--         value, age (years) between two dates
*-- Usage:    age2( dDate1, dDate2 )
*-- Returns:  numeric
#ifndef AGE2
   #define AGE2(dDate1,dDate2) ( floor( ( val( dtos(dDate2) ) ;
                                - val( dtos(dDate1) ) ) / 10000 ) )
#endif

*---------------------------------------------------------------------
*-- DateText -- Miriam Liskin created original version of this years
*--             ago, "lifted" by Ken M., and turned into inline by
*--             Jay Parsons
*-- Usage:  DateText( dDate )
*-- Returns: character - date in month day, year format:
*--                      i.e.,  February 15, 1997
#ifndef DATETEXT
   #define DATETEXT( dDate ) ( cmonth( dDate ) +" "+;
                             ltrim(str(day( dDate ),2))+", "+;
                             str(year( dDate ),4) )
#endif

*---------------------------------------------------------------------
*-- DateText2 -- Miriam Liskin created original version of this years
*--              ago, "lifted" by Ken M., and turned into inline by
*--              Jay Parsons (This one requires DateText above)
*-- Usage:   DateText2( dDate )
*-- Returns: character - date in character day, month day, year format:
*--                      i.e.,  Saturday, February 15, 1997
#ifndef DATETEXT2
   #define DATETEXT2(dDate) ( cdow(dDate ) +  ", " + DATETEXT(dDate ))
#endif

*---------------------------------------------------------------------
*-- DateText3 -- Miriam Liskin created original version of this years
*--              ago, "lifted" by Ken M., and turned into inline by
*--              Jay Parsons -- this is Doug Seine's version -- he 
*--              removed the day of month bits
*-- Usage:   DateText3( dDate )
*-- Returns: character - date in month day, year format:
*--                      i.e.,  February 1997
#ifndef DATETEXT3
    #define DATETEXT3(dDate) ( cmonth(dDate)+", "+str(year(dDate),4) )
#endif

*---------------------------------------------------------------------
*-- Date2Jul -- Jay Parsons -- returns # of days since January
*--             1, 4713 B.C. (Astronomer's "Julian" date)
*--             Do not use for dBASE dates before 1582, or without 
*--             considering time zones if using for astronomy.
*-- Usage:   Date2Jul( dDate )
*-- Returns: numeric - # of days ...
#ifndef DAT2JUL
   #define DAT2JUL(dDate) ( 2415386 + dDate - ctod( "01/01/01" ) )
#endif

*---------------------------------------------------------------------
*-- Jul2Date -- Jay Parsons -- Converts Julian # of days to dBASE Date
*--             (Opposite of Date2Jul)
*-- Usage:   Jul2Date( nDays )
*-- Returns: Date
#ifndef JUL2DAT
   #define JUl2DAT(nDays) ( ctod( "01/01/01" ) + (nDays) - 2415386)
#endif

*---------------------------------------------------------------------
*-- FDoM -- Ken Chan -- this returns the first day of the month for
*--                     the month of a specific date.
*-- Usage:   FDoM( dDate )
*-- Returns: Date
#ifndef FDOM
   #define FDOM( dDate )  ( ( dDate ) - day( dDate ) + 1 )
#endif

*---------------------------------------------------------------------
*-- FRSTNxtMth -- Todd Barry -- this returns the first day of the 
*--               next month for the month of a specific date.
*-- Usage:   FRSTNxtMth( dDate )
*-- Returns: Date
#ifndef FRSTNXTMTH
   #define FRSTNXTMTH(dDate)  ( FDOM( ( dDate ) - day( dDate ) + 45 ) )
#endif

*---------------------------------------------------------------------
*-- NDoW -- Jay Parsons -- returns numeric day of week for text
*--         version.
*-- Usage:   NDoW( cDay )
*-- Returns: Numeric
#ifndef NDOW
   #define NDOW(cDay)   ( at( upper( left( ltrim( cDay ), 3 ) ), ;
                        "   SUN MON TUE WED THU FRI SAT" ) / 4 )
#endif

*---------------------------------------------------------------------
*-- Quarter -- Bowen Moursund -- returns quarter of year a date falls
*--            in
*-- Usage:   Quarter( dDate )
*-- Returns: numeric
#ifndef QUARTER
   #define QUARTER(dDate) ( iif( type( "dDate" )="D",;
                            ceiling( month( dDate ) /3 ),0 ) )
#endif


*----------------------------------------------------------------------
*-- string routines
*----------------------------------------------------------------------
*-- AllTrim -- lots of sources ... trims on both sides of string.
*-- Usage:   alltrim( cString )
*-- Returns: character
#define alltrim( cString )   ltrim(rtrim( cString ))

*----------------------------------------------------------------------
*-- LastWord -- Martin Leon - returns last word of character string
*--             (assumes spaces between words)
*-- Usage:   LastWord( cString )
*-- Returns: character
#ifndef LASTWORD
   #define LASTWORD(cString) ( substr( (cString), ;
                             rat( " ", trim(cString) ) + 1 ) )
#endif

*----------------------------------------------------------------------
*-- field or table routines
*----------------------------------------------------------------------
*-- Scatter/Gather -- based on request from the VDBASE Forum on 
*-- compuserve, these are simple versions that do not handle binary
*-- fields -- any binary (memo, binary, ole) fields in a table will 
*-- have '.f.' placed into the array element -- suggest you do not
*-- use these routines with tables that contain memo, binary, or
*-- ole fields. Inline versions created by Romain Streiff.
*----------------------------------------------------------------------
*-- scatter places contents of current record into an array 
*-- (_app.aScatter). 
*-- Usage:   scatter
*-- Returns: null
#ifndef SCATTER
   #define scatter _app.aScatter=new Array( fldcount()); 
                   copy to array _app.aScatter next 1
#endif
 
*-- gather replaces the current record's contents with the contents
*-- of the array _app.aScatter, assuming it exists.
*-- Usage:   gather
*-- Returns: null
#ifndef GATHER
   #define gather if type("_app.aScatter") == "A"; 
                  replace from array _app.aScatter next 1; 
                  endif
#endif

*----------------------------------------------------------------------
*-- Used -- Ken Mayer -- returns true or false values based on whether
*--         or not a table is in use at the moment.
*-- Usage:   Used( cTableName )  && or alias name
*-- Returns: logical
#ifndef USED 
   #define USED( cAlias ) ( select( cAlias ) # 0 ) 
#endif 

*-----------------------------------------------------------------------
*-- misc routines:

*-----------------------------------------------------------------------
*-- Beep -- Created by Bowen Moursund, simply sounds the 'bell' n times.
*-- Usage:   beep(<n>)
*-- Returns: null
#ifndef BEEP
   #define beep(n) ?? replicate( chr(7), n)
#endif

*-----------------------------------------------------------------------
*-- RandNum -- Ken Mayer -- Random number between 1 and number supplied
*--            If used often, suggest one time per run of an application
*--            call to RAND( -1 ) to seed random number table.
*-- Usage:   RandNum( nMax )
*-- Returns: numeric
#ifndef RandNum 
    #define RandNum( nMax ) ( int( rand() * ( nMax ) ) + 1 ) 
#endif 


*-----------------------------------------------------------------------
*-- End of File: dBLOAT.H
*-----------------------------------------------------------------------