*-- File: SYSCOLOR.CC

CLASS SysColor
*----------------------------------------------------------------------
*-- Description:
*
*     Assigns and retrieves system colors for the standard window
*     components.
*
*-- Programmers:
*
*     Jim Sare (CIS: 71333,1030)
*     
*-- History: 
*     10/31/1994 -- Original
*     06/21/1995 -- Added/updated comments.
*     12/01/1996 -- original version of these routines as a class.
*
*-- Instantiation:
*
*     set procedure to syscolor.cc additive
*     _app.SysColor = new syscolor()
*
*-- Methods:
*     Note, more details are in the headers for each method, including
*     a listing of parameters, and what they mean, as well as the
*     complete history (as we know it) for each method.
*
*-- Example of use:
*     _app.SysColor.OldActiveCaption = GetColors( COLOR_ACTIVECAPTION ) 
*                                        && save color of title bar
*     _app.SysColor.SetColors(COLOR_ACTIVECAPTION,HTOI("00FF00")) 
*                                        && make title bar green
*     *-- code here
*     _app.SysColor.SetColors(COLOR_ACTIVECAPTION, OldActiveCaption)  
*                                        && restore original
*----------------------------------------------------------------------

   *-- These are index DEFINEs for GetColors() and SetColors()
   #define COLOR_SCROLLBAR            0
   #define COLOR_BACKGROUND           1
   #define COLOR_ACTIVECAPTION        2
   #define COLOR_INACTIVECAPTION      3
   #define COLOR_MENU                 4
   #define COLOR_WINDOW               5
   #define COLOR_WINDOWFRAME          6
   #define COLOR_MENUTEXT             7
   #define COLOR_WINDOWTEXT           8
   #define COLOR_CAPTIONTEXT          9
   #define COLOR_ACTIVEBORDER        10
   #define COLOR_INACTIVEBORDER      11
   #define COLOR_APPWORKSPACE        12
   #define COLOR_HIGHLIGHT           13
   #define COLOR_HIGHLIGHTTEXT       14
   #define COLOR_BTNFACE             15
   #define COLOR_BTNSHADOW           16
   #define COLOR_GRAYTEXT            17
   #define COLOR_BTNTEXT             18
   #define COLOR_INACTIVECAPTIONTEXT 19
   #define COLOR_BTNHIGHLIGHT        20

   *-- MAKE16 and MAKE32 are adaptations of Jay Parsons' MAKEWORD and 
   *-- MAKELONG.
   #ifndef MAKE16
      #define MAKE16(n) (chr(bitand((n),255)) + chr(bitrshift((n),8)))
   #endif

   #ifndef MAKE32
      #define MAKE32(n) (MAKE16(bitand((n),65535)) + ;
                         MAKE16(bitrshift((n),16)))
   #endif

   FUNCTION SetColors(nIndex, nColor)
   *-------------------------------------------------------------------
   *-- Programmer..: Jim Sare (CIS: 71333,1030)
   *-- Date........: 06/21/1995
   *-- Notes.......: Assigns colors to the standard window components. 
   *--               The color changes are system-wide. The changes 
   *--               will be reflected in all applications currently 
   *--               running or started after the new color assignment.
   *--               The changes are not permanent. The colors will be 
   *--               restored to their previous values when the system
   *--               is restarted.
   *-- Written for.: dBASE for Windows (any version)
   *-- Rev. History: 10/31/1994 -- Original
   *--               06/21/1995 -- Added/updated comments.
   *-- Calls.......: None
   *--               WINAPI: SetSysColors
   *-- Usage.......: SysColor.SetColor(2, HTOI("00FF00") 
   *--                                           && Make title bar 
   *--                                           && color green
   *-- Returns.....: None
   *-- Parameters..: nIndex = Numeric index for the component you want 
   *--                        to change.
   *--                        (see #define values above.)
   *--               nColor = Color value to assign to the component. 
   *--                        The Color parameter to SetColors is 
   *--                        standard RGB format in reverse. ie;BBGGRR
   *--                        using HTOI() to convert the hex value to 
   *--                        numeric.
   *-------------------------------------------------------------------

      if type("SetSysColors") # "FP"
         extern CVOID SetSysColors(CINT, CPTR, CPTR) User
      endif

      SetSysColors(1, MAKE32(nIndex), MAKE32(nColor))
   RETURN NULL
   *-- EOM: SetColors

   FUNCTION GetColors(nIndex)
   *-------------------------------------------------------------------
   *-- Programmer..: Jim Sare (CIS: 71333,1030)
   *-- Date........: 06/21/1995
   *-- Notes.......: Retrieves current color of the standard window 
   *--               components.
   *-- Written for.: dBASE for Windows (any version)
   *-- Rev. History: 10/31/1994 -- Original
   *--               06/21/1995 -- Added/updated comments.
   *-- Calls.......: None
   *--               WINAPI: SetSysColors
   *-- Usage.......: Set Procedure to SysColor.cc Additive
   *--               OldActiveCaption = _app.SysColor.GetColors(2) 
   *--                                               && Save active 
   *--                                               && title bar color
   *--               _app.SysColor.SetColors(2, HTOI("00FF00")
   *--                                               && Make title bar 
   *--                                               && color green
   *--               * Do whatever else here. When ready to restore 
   *--               * original color
   *--               _app.SysColor.SetColors(2, OldActiveCaption)  
   *--                                        && Restore original color
   *-- Returns.....: Numeric color value of specified component.
   *-- Parameters..: nIndex = Numeric index for the component you want 
   *--                        to obtain the color of.
   *--                        (see #define values above.)
   *-------------------------------------------------------------------

   if type("GetSysColor") # "FP"
      extern CLONG GetSysColor(CINT) User
   endif

   RETURN GetSysColor(nIndex)
   *-- EoM: SetColors

ENDCLASS
*-- End of Class: SysColor

*-----------------------------------------------------------------------
*-- End of File: SYSCOLOR.CC
*-----------------------------------------------------------------------
